import React, { useRef, useEffect } from 'react';
import ReactDOM from 'react-dom';
import Tooltip from './components/Tooltip';
// eslint-disable-next-line import/no-webpack-loader-syntax
import vjmap from '!vjmap'
import 'vjmap/dist/vjmap.min.css'
import "./Map.css"

const Map = () => {
    const mapContainerRef = useRef(null);
    const tooltipRef = useRef(new vjmap.Popup({ offset: 15 }));

    // 初始化
    // eslint-disable-next-line react-hooks/exhaustive-deps
    useEffect(async () => {
        const env = {
            serviceUrl: "https://vjmap.com/server/api/v1",
            accessToken: "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJJRCI6MSwiVXNlcm5hbWUiOiJyb290MSIsIk5pY2tOYW1lIjoicm9vdDEiLCJBdXRob3JpdHlJZCI6InJvb3QiLCJCdWZmZXJUaW1lIjo4NjQwMCwiZXhwIjoxOTQyMzA1MjQxLCJpc3MiOiJ2am1hcCIsIm5iZiI6MTYyNjk0NDI0MX0.29OAA4Cnjtw780VxIxWqduLtszp1EtpSNHAmWjiL_OM",
            exampleMapId: "sys_zp"
        };
        // 地图服务对象
        let svc = new vjmap.Service(env.serviceUrl, env.accessToken)
        // 打开地图
        let res = await svc.openMap({
            mapid: env.exampleMapId, // 地图ID
            mapopenway: vjmap.MapOpenWay.GeomRender, // 以几何数据渲染方式打开
            style: vjmap.openMapDarkStyle() // div为深色背景颜色时，这里也传深色背景样式
        })
        if (res.error) {
            // 如果打开出错
            console.error(res.error)
            return;
        }
        // 获取地图范围
        let mapExtent = vjmap.GeoBounds.fromString(res.bounds);
        // 根据地图范围建立几何投影坐标系
        let prj = new vjmap.GeoProjection(mapExtent);

        // 地图对象
        let map = new vjmap.Map({
            container: mapContainerRef.current, // DIV容器ID
            style: svc.vectorStyle(), // 样式，这里是栅格样式
            center: prj.toLngLat(mapExtent.center()), // 设置地图中心点
            zoom: 1, // 设置地图缩放级别,
            pitch: 0, // 倾斜角度
            renderWorldCopies: false // 不显示多屏地图
        });

        // 关联服务对象和投影对象
        map.attach(svc, prj);
        // 根据地图本身范围缩放地图至全图显示
        //map.fitMapBounds();
        await map.onLoad(); // 等待地图加载完成

        // 获取所有图层
        const layers = svc.getMapLayers();
        // 实体类型ID和名称映射
        const { entTypeIdMap } = await svc.getConstData();

        let mousePositionControl = new vjmap.MousePositionControl();
        map.addControl(mousePositionControl, "bottom-left");


        // 有高亮状态（鼠标在地图元素上时，会高亮)
        map.enableVectorLayerHoverHighlight((eventName, feature, layer, e) => {
            if (eventName === "mouseleave") {
                tooltipRef.current.remove();
                return;
            }
            // 点击高亮实体回调事件
            const prop = feature.properties;
            if (!feature || !prop) return
            prop.layerName = layers[prop.layer].name;
            prop.typeName = entTypeIdMap[prop.type];
            prop.id = feature.id;

            // 创建提示节点
            const tooltipNode = document.createElement('div');
            ReactDOM.render(<Tooltip prop={prop} />, tooltipNode);

            // 设置tooltip
            tooltipRef.current
                .setLngLat(e.lngLat)
                .setDOMContent(tooltipNode)
                .addTo(map);
        })
       

        // 退出移除
        return () => map.remove();
    }, []);

    return (
        <div>
            <div className='map-container' ref={mapContainerRef} />
        </div>
    );
};

export default Map;